==> BAMBI_1.0.0.tar.gz: BAMBI/src/vmcos_llik_grad.cpp <==
#include <RcppArmadillo.h>
// [[Rcpp::depends(RcppArmadillo)]]

#include "bessel.h"
#include "univmgen.h"

using namespace Rcpp;

typedef std::vector<arma::mat> stdvec_mat;
typedef std::vector<arma::vec> stdvec_vec;



// Returns the modified Bessel function I_0 (x) for any real x
// same as the original function in bessel.h, but this one exports to R
// [[Rcpp::export]]
double BESSI0_C(double x)
{
  double ax,ans=0;
  double y;
  if ((ax=fabs(x)) < 3.75) { // Polynomial fit
    y=x/3.75;
    y*=y;
    ans=1.0+y*(3.5156229+y*(3.0899424+y*(1.2067492
                                           +y*(0.2659732+y*(0.360768e-1+y*0.45813e-2)))));
  } else {
    y=3.75/ax;
    ans=(exp(ax)/sqrt(ax))*(0.39894228+y*(0.1328592e-1
                                            +y*(0.225319e-2+y*(-0.157565e-2+y*(0.916281e-2
                                                                                 +y*(-0.2057706e-1+y*(0.2635537e-1+y*(-0.1647633e-1
                                                                                 +y*0.392377e-2))))))));
  }
  return ans;
}




// [[Rcpp::export]]
double const_vmcos_anltc(double k1, double k2, double k3)
{
  int p = 1;
  double start, temp;
  start = BESSI0(k1) * BESSI0(k2) * BESSI0(k3);
  temp = 2 * BESSI(p, k1) * BESSI(p, k2) * BESSI(p, k3);
  start += temp;
  while((fabs(temp) > 1e-7) && (p <= 10000)) {
    p++;
    temp = 2 * BESSI(p, k1) * BESSI(p, k2) * BESSI(p, k3);
    start += temp;
  }
  return  4 * M_PI * M_PI * start;
}


// [[Rcpp::export]]
double const_vmcos_mc(double k1, double k2, double k3, arma::mat uni_rand, int ncores = 1)
{
  double x = uni_rand(0, 0) * 2 * M_PI,
    y = uni_rand(0, 1) * 2 * M_PI,
    temp = k1 * cos(x) + k2 * cos(y) + k3 * cos(x - y),
    expon, sum_exp = 1.0;
  int nsim = uni_rand.n_rows, i;

#pragma omp parallel for reduction(+:sum_exp) private(expon, x, y) num_threads(ncores)
  for(i = 1; i < nsim; i++) {
    x = uni_rand(i, 0) * 2 * M_PI;
    y = uni_rand(i, 1) * 2 * M_PI;
    expon = k1 * cos(x) + k2 * cos(y) + k3 * cos(x - y);
    sum_exp += exp(expon - temp);
  }
  return 4 * M_PI * M_PI * exp(temp) * sum_exp / nsim;
}



// [[Rcpp::export]]
double const_vmcos(double k1, double k2, double k3, arma::mat  uni_rand, int ncores = 1)
{
  if((k3 >= -5) || (fmax(fmax(k1, k2), -k3) <= 15)) {
    return const_vmcos_anltc(k1, k2, k3);
  } else {
    return const_vmcos_mc(k1, k2, k3, uni_rand, ncores);
  }
}

// [[Rcpp::export]]
double d_const_vmcos_k1_anltc(double k1, double k2, double k3)
{
  int p = 1;
  double start, temp;
  start = BESSI1(k1) * BESSI0(k2) * BESSI0(k3);
  temp = (BESSI(p-1, k1) + BESSI(p+1, k1))  * BESSI(p, k2) * BESSI(p, k3);
  start += temp;
  while((fabs(temp) > 1e-7) && (p <= 10000)) {
    p++;
    temp = (BESSI(p-1, k1) + BESSI(p+1, k1))  * BESSI(p, k2) * BESSI(p, k3);
    start += temp;
  }
  return  4 * M_PI * M_PI * start;
}


// [[Rcpp::export]]
arma::vec d_const_vmcos_anltc(double k1, double k2, double k3)
{
  arma::vec result;
  result << d_const_vmcos_k1_anltc(k1, k2, k3)
         << d_const_vmcos_k1_anltc(k2, k3, k1)
         << d_const_vmcos_k1_anltc(k3, k1, k2)
         << arma::endr;
  return result;
}



// [[Rcpp::export]]
arma::vec d_const_vmcos_mc(double k1, double k2, double k3, arma::mat uni_rand, int ncores = 1)
{
  double x = uni_rand(0, 0) * 2 * M_PI,
    y = uni_rand(0, 1) * 2 * M_PI,
    expon, sum_exp1 = cos(x), sum_exp2 = cos(y), sum_exp3 = cos(x-y),
    temp = k1 * sum_exp1 + k2 * sum_exp2 + k3 * sum_exp3;

  int nsim = uni_rand.n_rows, i;

#pragma omp parallel for reduction(+:sum_exp1, sum_exp2, sum_exp3) private(expon, x, y) num_threads(ncores)
  for(i = 1; i < nsim; i++) {
    x = uni_rand(i, 0) * 2 * M_PI;
    y = uni_rand(i, 1) * 2 * M_PI;
    expon = k1 * cos(x) + k2 * cos(y) + k3 * cos(x - y);
    sum_exp1 += exp(expon - temp) * cos(x);
    sum_exp2 += exp(expon - temp) * cos(y);
    sum_exp3 += exp(expon - temp) * cos(x-y);
  }
  arma::vec result;
  result << 4 * M_PI * M_PI * exp(temp) * sum_exp1 / nsim
         << 4 * M_PI * M_PI * exp(temp) * sum_exp2 / nsim
         << 4 * M_PI * M_PI * exp(temp) * sum_exp3 / nsim << arma::endr;
  return result;
}


// [[Rcpp::export]]
arma::vec d_const_vmcos(arma::vec par, arma::mat uni_rand, int ncores = 1)
{
  double k1 = par[0], k2 = par[1], k3 = par[2];
  if((k3 >= -5) || (fmax(fmax(k1, k2), -k3) <= 15)) {
    return d_const_vmcos_anltc(k1, k2, k3);
  } else {
    return d_const_vmcos_mc(k1, k2, k3, uni_rand, ncores);
  }
}


// [[Rcpp::export]]
arma::vec log_const_vmcos_all(arma::mat par_mat, arma::mat uni_rand, int ncores = 1) {
  int K = par_mat.n_cols;
  arma::vec all_lconsts(K);
  for(int j = 0; j < K; j++)
    all_lconsts[j] = log(const_vmcos(par_mat(0,j), par_mat(1,j), par_mat(2,j), uni_rand, ncores));
  return all_lconsts;
}





// [[Rcpp::export]]
double ldcosnum(double x, double y, arma::vec par)
{
  double k1 = par[0], k2 = par[1], k3 = par[2], mu = par[3], nu = par[4];
  if(k1 <= 0 || k2 <= 0 )
    return NA_REAL; else
      return (k1*cos(x-mu) + k2*cos(y-nu) + k3 * cos(x-mu-y+nu));
}

// [[Rcpp::export]]
arma::vec dcos_onex_manypar(arma::vec x, arma::vec k1, arma::vec k2, arma::vec k3,
                            arma::vec mu1, arma::vec mu2, arma::vec l_const_all)
{
  int n = k1.size();

  arma::mat all_par(5, n);
  for(int i = 0; i < n; i++) {
    all_par(0,i) = k1[i];
    all_par(1,i) = k2[i];
    all_par(2,i) = k3[i];
    all_par(3,i) = mu1[i];
    all_par(4,i) = mu2[i];
  }

  arma::vec ld_num(n);
  for(int i = 0; i < n; i++) {
    ld_num[i] = ldcosnum(x[0], x[1], all_par.col(i));
  }

  return arma::exp(ld_num - l_const_all);
}

// [[Rcpp::export]]
arma::vec dcos_manyx_onepar(arma::mat x, double k1, double k2, double k3,
                            double mu1, double mu2, double l_const)
{
  int n = x.n_rows;

  arma::vec par(5);
  par[0] = k1; par[1] = k2; par[2] = k3; par[3] = mu1; par[4] = mu2;

  arma::vec ld_num(n);
  for(int i = 0; i < n; i++) {
    ld_num[i] = ldcosnum(x(i,0), x(i,1), par);
  }

  return arma::exp(ld_num - l_const);
}


// [[Rcpp::export]]
arma::vec dcos_manyx_manypar(arma::mat x, arma::vec k1, arma::vec k2, arma::vec k3,
                             arma::vec mu1, arma::vec mu2, arma::vec l_const_all)
{
  int n = k1.size();

  arma::mat all_par(5, n);
  for(int i = 0; i < n; i++) {
    all_par(0,i) = k1[i];
    all_par(1,i) = k2[i];
    all_par(2,i) = k3[i];
    all_par(3,i) = mu1[i];
    all_par(4,i) = mu2[i];
  }

  arma::vec ld_num(n);
  for(int i = 0; i < n; i++) {
    ld_num[i] = ldcosnum(x(i,0), x(i,1), all_par.col(i));
  }

  return arma::exp(ld_num - l_const_all);
}



// generates a single observation
arma::rowvec rcos_single_onepar(double k1, double k2, double k3,
                                double mu1, double mu2, double I_upper_bd) {
  arma::vec par(5);
  par << k1 << k2 << k3 << mu1 << mu2 << arma::endr;

  double x, y, U1, k13;

  // First generate y from the marginal distribution given in Eqn. 5 in Mardia 2007

  int accpt_y = 0;

  while (accpt_y < 1) {
    y = runivm_single_onepar(k2, mu2);
    k13 = sqrt(k1 * k1 + k3 * k3 + 2 * k1 * k3 * cos(y - mu2));
    U1 = R::unif_rand();
    // I_upper_bd is numerically calculated in R
    if (U1 <= BESSI0(k13) / I_upper_bd)
      accpt_y++;
  }

  double y_mu2 = atan( k3 * sin(y - mu2) / (k1 + k3 * cos(y - mu2)) );

  // Now generate x from vM(k13, mu1 + y_mu2)
  x = runivm_single_onepar(k13, (mu1 + y_mu2));

  arma::rowvec final_sample(2);
  final_sample[0] = x;
  final_sample[1] = y;

  return final_sample;
}



//  [[Rcpp::export]]
arma::mat rcos_onepar(int n, double k1, double k2, double k3,
                      double mu1, double mu2, double I_upper_bd) {
  if(n == 1){
    return(rcos_single_onepar(k1, k2, k3, mu1, mu2, I_upper_bd));
  } else {
    arma::mat all_sample(n, 2);
    for(int i = 0; i < n; i++)
      all_sample.row(i) = rcos_single_onepar(k1, k2, k3, mu1, mu2, I_upper_bd);
    return(all_sample);
  }
}


//  [[Rcpp::export]]
arma::mat rcos_manypar(arma::vec k1, arma::vec k2, arma::vec k3,
                       arma::vec mu1, arma::vec mu2, arma::vec I_upper_bd) {
  int n = k1.size();
  arma::mat all_sample(n, 2);
  for(int i = 0; i < n; i++)
    all_sample.row(i) = rcos_single_onepar(k1[i], k2[i], k3[i], mu1[i], mu2[i], I_upper_bd[i]);
  return(all_sample);
}


// [[Rcpp::export]]
arma::mat mem_p_cos(arma::mat data, arma::mat par, arma::vec pi,
                    arma::vec log_c_von, int ncores = 1)
{
  int n = data.n_rows, K = par.n_cols, i, j;
  double row_total;
  arma::mat den(n, K);
#pragma omp parallel for private(j, row_total) shared(i) num_threads(ncores)
  for(i = 0; i < n; i++){
    row_total = 0;
    for(j = 0; j < K; j++){
      den(i, j) = pi[j]*exp(ldcosnum(data(i, 0), data(i, 1), par.col(j)) - log_c_von[j]);;
      row_total += den(i, j);
    }
    if(row_total < 1e-50)
      row_total = 1e-50;
    for(j = 0; j < K; j++)
      den(i, j) /= row_total;
  }
  return(den);
}


// [[Rcpp::export]]
double llik_vmcos_full(arma::mat data, arma::mat par, arma::vec pi, arma::vec log_c, int ncores = 1)
{

  int n = data.n_rows, K = pi.size(), i ,j;
  long double temp, log_sum = 0.0;
  arma::vec log_pi = log(pi);
  if(K > 1) {
#pragma omp parallel for reduction(+:log_sum)  private(j, temp) num_threads(ncores)
    for(i = 0; i < n; i++) {
      temp = 0;
      for(j = 0; j < K; j++)
        temp += exp(ldcosnum(data(i,0), data(i,1), par.col(j)) - log_c[j] + log_pi[j] );
      log_sum += log(temp);
    }
  } else {
#pragma omp parallel for reduction(+:log_sum) shared(i)  num_threads(ncores)
    for(i = 0; i < n; i++)
      log_sum += ldcosnum(data(i,0), data(i,1), par);
    log_sum -= n*log_c[0];
  }

  return(log_sum);

}


// [[Rcpp::export]]
arma::vec grad_log_vmcos_one_comp_i(double x, double y, arma::vec par,
                                    double c_vmcos, arma::vec del_const_vmcos)
{
  double k1 = par[0], k2 = par[1], k3 = par[2], mu1 = par[3], mu2 = par[4];
  arma::vec all_entries(5);
  all_entries[0] = (cos(x - mu1) - del_const_vmcos[0]/c_vmcos);
  all_entries[1] = (cos(y - mu2) - del_const_vmcos[1]/c_vmcos);
  all_entries[2] = (cos(x - mu1 - y + mu2) - del_const_vmcos[2]/c_vmcos);
  all_entries[3] = (k1*sin(x - mu1) + k3*sin(x - mu1 - y + mu2));
  all_entries[4] = (k2*sin(y - mu2) - k3*sin(x - mu1 - y + mu2));

  return all_entries;
}


// [[Rcpp::export]]
arma::mat grad_vmcos_all_comp(arma::mat data, arma::mat par_mat,
                              arma::vec pi, arma::mat uni_rand, int ncores = 1)
{
  int n = data.n_rows, K = pi.size(), i ,j;
  arma::vec l_c_vmcos = log_const_vmcos_all(par_mat, uni_rand, ncores);
  arma::vec c_vmcos = arma::exp(l_c_vmcos), log_pi = log(pi);

  stdvec_vec del_const_vmcos_all(K);
  stdvec_vec par_mat_cols(K);

  for(j = 0; j < K; j++) {
    del_const_vmcos_all[j] = d_const_vmcos(par_mat.col(j), uni_rand, ncores);
    par_mat_cols[j] = par_mat.col(j);
  }

  stdvec_mat grad_sum_part(ncores);
  for(int g = 0; g < ncores; g++)
    grad_sum_part[g] = arma::zeros(5, K);

  double denom, temp;

#pragma omp parallel for   private(j, denom, temp) num_threads(ncores)
  for(i = 0; i < n; i++) {
    arma::mat grad_temp(5, K);
    int g = omp_get_thread_num();
    // int g = 0;
    denom = 0;
    for(j = 0; j < K; j++) {
      temp = exp(ldcosnum(data(i, 0), data(i, 1), par_mat_cols[j]) - l_c_vmcos[j] + log_pi[j]);
      denom += temp;
      grad_temp.col(j) = temp*grad_log_vmcos_one_comp_i(data(i, 0), data(i, 1), par_mat_cols[j], c_vmcos[j], del_const_vmcos_all[j]);
    }
    grad_sum_part[g] += grad_temp/denom;
  }

  arma::mat grad_sum = arma::zeros(5, K);
  for(int g = 0; g < ncores; g++)
    grad_sum += grad_sum_part[g];

  return (grad_sum);
}



// [[Rcpp::export]]
double vmcosmix(double x, double y, arma::mat par, arma::vec pi, arma::vec log_c_von)
{
  double res = 0;
  int K = par.n_cols;

  for(int j = 0; j < K; j++)
    res += exp(ldcosnum(x, y, par.col(j)) - log_c_von[j]) * pi[j];
  return(res);
} //likelihood contribution (density) of a single point in a mixture model

// [[Rcpp::export]]
arma::vec vmcosmix_manyx(arma::mat x, arma::mat par, arma::vec pi, arma::vec log_c_von)
{
  int n = x.n_rows;
  arma::vec result(n);
  for(int i = 0; i < n; i++)
    result[i] = vmcosmix(x(i,0), x(i,1), par, pi, log_c_von);
  return(result);
}




// [[Rcpp::export]]
void hellomp()
{
  // thread id and number of threads
  int tid, nthreads;

#pragma omp parallel private(tid)
{
  nthreads = omp_get_num_threads();
  tid = omp_get_thread_num();

  Rprintf("Hello from thread %d of %d\n", tid, nthreads);
}
}

==> BAMBI_1.0.1.tar.gz: BAMBI/src/vmcos_llik_grad.cpp <==
#include <RcppArmadillo.h>
// [[Rcpp::depends(RcppArmadillo)]]


#ifdef _OPENMP
#include <omp.h>
#endif


#include "thread_num.h"
#include "bessel.h"
#include "univmgen.h"

using namespace Rcpp;

typedef std::vector<arma::mat> stdvec_mat;
typedef std::vector<arma::vec> stdvec_vec;



// Returns the modified Bessel function I_0 (x) for any real x
// same as the original function in bessel.h, but this one exports to R
// [[Rcpp::export]]
double BESSI0_C(double x)
{
  double ax,ans=0;
  double y;
  if ((ax=fabs(x)) < 3.75) { // Polynomial fit
    y=x/3.75;
    y*=y;
    ans=1.0+y*(3.5156229+y*(3.0899424+y*(1.2067492
                                           +y*(0.2659732+y*(0.360768e-1+y*0.45813e-2)))));
  } else {
    y=3.75/ax;
    ans=(exp(ax)/sqrt(ax))*(0.39894228+y*(0.1328592e-1
                                            +y*(0.225319e-2+y*(-0.157565e-2+y*(0.916281e-2
                                                                                 +y*(-0.2057706e-1+y*(0.2635537e-1+y*(-0.1647633e-1
                                                                                 +y*0.392377e-2))))))));
  }
  return ans;
}




// [[Rcpp::export]]
double const_vmcos_anltc(double k1, double k2, double k3)
{
  int p = 1;
  double start, temp;
  start = BESSI0(k1) * BESSI0(k2) * BESSI0(k3);
  temp = 2 * BESSI(p, k1) * BESSI(p, k2) * BESSI(p, k3);
  start += temp;
  while((fabs(temp) > 1e-7) && (p <= 10000)) {
    p++;
    temp = 2 * BESSI(p, k1) * BESSI(p, k2) * BESSI(p, k3);
    start += temp;
  }
  return  4 * M_PI * M_PI * start;
}


// [[Rcpp::export]]
double const_vmcos_mc(double k1, double k2, double k3, arma::mat uni_rand, int ncores = 1)
{
  double x = uni_rand(0, 0) * 2 * M_PI,
    y = uni_rand(0, 1) * 2 * M_PI,
    temp = k1 * cos(x) + k2 * cos(y) + k3 * cos(x - y),
    expon, sum_exp = 1.0;
  int nsim = uni_rand.n_rows, i;

#pragma omp parallel for reduction(+:sum_exp) private(expon, x, y) num_threads(ncores)
  for(i = 1; i < nsim; i++) {
    x = uni_rand(i, 0) * 2 * M_PI;
    y = uni_rand(i, 1) * 2 * M_PI;
    expon = k1 * cos(x) + k2 * cos(y) + k3 * cos(x - y);
    sum_exp += exp(expon - temp);
  }
  return 4 * M_PI * M_PI * exp(temp) * sum_exp / nsim;
}



// [[Rcpp::export]]
double const_vmcos(double k1, double k2, double k3, arma::mat  uni_rand, int ncores = 1)
{
  if((k3 >= -5) || (fmax(fmax(k1, k2), -k3) <= 15)) {
    return const_vmcos_anltc(k1, k2, k3);
  } else {
    return const_vmcos_mc(k1, k2, k3, uni_rand, ncores);
  }
}

// [[Rcpp::export]]
double d_const_vmcos_k1_anltc(double k1, double k2, double k3)
{
  int p = 1;
  double start, temp;
  start = BESSI1(k1) * BESSI0(k2) * BESSI0(k3);
  temp = (BESSI(p-1, k1) + BESSI(p+1, k1))  * BESSI(p, k2) * BESSI(p, k3);
  start += temp;
  while((fabs(temp) > 1e-7) && (p <= 10000)) {
    p++;
    temp = (BESSI(p-1, k1) + BESSI(p+1, k1))  * BESSI(p, k2) * BESSI(p, k3);
    start += temp;
  }
  return  4 * M_PI * M_PI * start;
}


// [[Rcpp::export]]
arma::vec d_const_vmcos_anltc(double k1, double k2, double k3)
{
  arma::vec result;
  result << d_const_vmcos_k1_anltc(k1, k2, k3)
         << d_const_vmcos_k1_anltc(k2, k3, k1)
         << d_const_vmcos_k1_anltc(k3, k1, k2)
         << arma::endr;
  return result;
}



// [[Rcpp::export]]
arma::vec d_const_vmcos_mc(double k1, double k2, double k3, arma::mat uni_rand, int ncores = 1)
{
  double x = uni_rand(0, 0) * 2 * M_PI,
    y = uni_rand(0, 1) * 2 * M_PI,
    expon, sum_exp1 = cos(x), sum_exp2 = cos(y), sum_exp3 = cos(x-y),
    temp = k1 * sum_exp1 + k2 * sum_exp2 + k3 * sum_exp3;

  int nsim = uni_rand.n_rows, i;

#pragma omp parallel for reduction(+:sum_exp1, sum_exp2, sum_exp3) private(expon, x, y) num_threads(ncores)
  for(i = 1; i < nsim; i++) {
    x = uni_rand(i, 0) * 2 * M_PI;
    y = uni_rand(i, 1) * 2 * M_PI;
    expon = k1 * cos(x) + k2 * cos(y) + k3 * cos(x - y);
    sum_exp1 += exp(expon - temp) * cos(x);
    sum_exp2 += exp(expon - temp) * cos(y);
    sum_exp3 += exp(expon - temp) * cos(x-y);
  }
  arma::vec result;
  result << 4 * M_PI * M_PI * exp(temp) * sum_exp1 / nsim
         << 4 * M_PI * M_PI * exp(temp) * sum_exp2 / nsim
         << 4 * M_PI * M_PI * exp(temp) * sum_exp3 / nsim << arma::endr;
  return result;
}


// [[Rcpp::export]]
arma::vec d_const_vmcos(arma::vec par, arma::mat uni_rand, int ncores = 1)
{
  double k1 = par[0], k2 = par[1], k3 = par[2];
  if((k3 >= -5) || (fmax(fmax(k1, k2), -k3) <= 15)) {
    return d_const_vmcos_anltc(k1, k2, k3);
  } else {
    return d_const_vmcos_mc(k1, k2, k3, uni_rand, ncores);
  }
}


// [[Rcpp::export]]
arma::vec log_const_vmcos_all(arma::mat par_mat, arma::mat uni_rand, int ncores = 1) {
  int K = par_mat.n_cols;
  arma::vec all_lconsts(K);
  for(int j = 0; j < K; j++)
    all_lconsts[j] = log(const_vmcos(par_mat(0,j), par_mat(1,j), par_mat(2,j), uni_rand, ncores));
  return all_lconsts;
}





// [[Rcpp::export]]
double ldcosnum(double x, double y, arma::vec par)
{
  double k1 = par[0], k2 = par[1], k3 = par[2], mu = par[3], nu = par[4];
  if(k1 <= 0 || k2 <= 0 )
    return NA_REAL; else
      return (k1*cos(x-mu) + k2*cos(y-nu) + k3 * cos(x-mu-y+nu));
}

// [[Rcpp::export]]
arma::vec dcos_onex_manypar(arma::vec x, arma::vec k1, arma::vec k2, arma::vec k3,
                            arma::vec mu1, arma::vec mu2, arma::vec l_const_all)
{
  int n = k1.size();

  arma::mat all_par(5, n);
  for(int i = 0; i < n; i++) {
    all_par(0,i) = k1[i];
    all_par(1,i) = k2[i];
    all_par(2,i) = k3[i];
    all_par(3,i) = mu1[i];
    all_par(4,i) = mu2[i];
  }

  arma::vec ld_num(n);
  for(int i = 0; i < n; i++) {
    ld_num[i] = ldcosnum(x[0], x[1], all_par.col(i));
  }

  return arma::exp(ld_num - l_const_all);
}

// [[Rcpp::export]]
arma::vec dcos_manyx_onepar(arma::mat x, double k1, double k2, double k3,
                            double mu1, double mu2, double l_const)
{
  int n = x.n_rows;

  arma::vec par(5);
  par[0] = k1; par[1] = k2; par[2] = k3; par[3] = mu1; par[4] = mu2;

  arma::vec ld_num(n);
  for(int i = 0; i < n; i++) {
    ld_num[i] = ldcosnum(x(i,0), x(i,1), par);
  }

  return arma::exp(ld_num - l_const);
}


// [[Rcpp::export]]
arma::vec dcos_manyx_manypar(arma::mat x, arma::vec k1, arma::vec k2, arma::vec k3,
                             arma::vec mu1, arma::vec mu2, arma::vec l_const_all)
{
  int n = k1.size();

  arma::mat all_par(5, n);
  for(int i = 0; i < n; i++) {
    all_par(0,i) = k1[i];
    all_par(1,i) = k2[i];
    all_par(2,i) = k3[i];
    all_par(3,i) = mu1[i];
    all_par(4,i) = mu2[i];
  }

  arma::vec ld_num(n);
  for(int i = 0; i < n; i++) {
    ld_num[i] = ldcosnum(x(i,0), x(i,1), all_par.col(i));
  }

  return arma::exp(ld_num - l_const_all);
}



// generates a single observation
arma::rowvec rcos_single_onepar(double k1, double k2, double k3,
                                double mu1, double mu2, double I_upper_bd) {
  arma::vec par(5);
  par << k1 << k2 << k3 << mu1 << mu2 << arma::endr;

  double x, y, U1, k13;

  // First generate y from the marginal distribution given in Eqn. 5 in Mardia 2007

  int accpt_y = 0;

  while (accpt_y < 1) {
    y = runivm_single_onepar(k2, mu2);
    k13 = sqrt(k1 * k1 + k3 * k3 + 2 * k1 * k3 * cos(y - mu2));
    U1 = R::unif_rand();
    // I_upper_bd is numerically calculated in R
    if (U1 <= BESSI0(k13) / I_upper_bd)
      accpt_y++;
  }

  double y_mu2 = atan( k3 * sin(y - mu2) / (k1 + k3 * cos(y - mu2)) );

  // Now generate x from vM(k13, mu1 + y_mu2)
  x = runivm_single_onepar(k13, (mu1 + y_mu2));

  arma::rowvec final_sample(2);
  final_sample[0] = x;
  final_sample[1] = y;

  return final_sample;
}



//  [[Rcpp::export]]
arma::mat rcos_onepar(int n, double k1, double k2, double k3,
                      double mu1, double mu2, double I_upper_bd) {
  if(n == 1){
    return(rcos_single_onepar(k1, k2, k3, mu1, mu2, I_upper_bd));
  } else {
    arma::mat all_sample(n, 2);
    for(int i = 0; i < n; i++)
      all_sample.row(i) = rcos_single_onepar(k1, k2, k3, mu1, mu2, I_upper_bd);
    return(all_sample);
  }
}


//  [[Rcpp::export]]
arma::mat rcos_manypar(arma::vec k1, arma::vec k2, arma::vec k3,
                       arma::vec mu1, arma::vec mu2, arma::vec I_upper_bd) {
  int n = k1.size();
  arma::mat all_sample(n, 2);
  for(int i = 0; i < n; i++)
    all_sample.row(i) = rcos_single_onepar(k1[i], k2[i], k3[i], mu1[i], mu2[i], I_upper_bd[i]);
  return(all_sample);
}


// [[Rcpp::export]]
arma::mat mem_p_cos(arma::mat data, arma::mat par, arma::vec pi,
                    arma::vec log_c_von, int ncores = 1)
{
  int n = data.n_rows, K = par.n_cols, j;
  double row_total;
  arma::mat den(n, K);
#pragma omp parallel for private(j, row_total) num_threads(ncores)
  for(int i = 0; i < n; i++){
    row_total = 0;
    for(j = 0; j < K; j++){
      den(i, j) = pi[j]*exp(ldcosnum(data(i, 0), data(i, 1), par.col(j)) - log_c_von[j]);;
      row_total += den(i, j);
    }
    if(row_total < 1e-50)
      row_total = 1e-50;
    for(j = 0; j < K; j++)
      den(i, j) /= row_total;
  }
  return(den);
}


// [[Rcpp::export]]
double llik_vmcos_full(arma::mat data, arma::mat par, arma::vec pi, arma::vec log_c, int ncores = 1)
{

  int n = data.n_rows, K = pi.size(), j;
  long double temp, log_sum = 0.0;
  arma::vec log_pi = log(pi);
  if(K > 1) {
#pragma omp parallel for reduction(+:log_sum)  private(j, temp) num_threads(ncores)
    for(int i = 0; i < n; i++) {
      temp = 0;
      for(j = 0; j < K; j++)
        temp += exp(ldcosnum(data(i,0), data(i,1), par.col(j)) - log_c[j] + log_pi[j] );
      log_sum += log(temp);
    }
  } else {
#pragma omp parallel for reduction(+:log_sum)  num_threads(ncores)
    for(int i = 0; i < n; i++)
      log_sum += ldcosnum(data(i,0), data(i,1), par);
    log_sum -= n*log_c[0];
  }

  return(log_sum);

}


// [[Rcpp::export]]
arma::vec grad_log_vmcos_one_comp_i(double x, double y, arma::vec par,
                                    double c_vmcos, arma::vec del_const_vmcos)
{
  double k1 = par[0], k2 = par[1], k3 = par[2], mu1 = par[3], mu2 = par[4];
  arma::vec all_entries(5);
  all_entries[0] = (cos(x - mu1) - del_const_vmcos[0]/c_vmcos);
  all_entries[1] = (cos(y - mu2) - del_const_vmcos[1]/c_vmcos);
  all_entries[2] = (cos(x - mu1 - y + mu2) - del_const_vmcos[2]/c_vmcos);
  all_entries[3] = (k1*sin(x - mu1) + k3*sin(x - mu1 - y + mu2));
  all_entries[4] = (k2*sin(y - mu2) - k3*sin(x - mu1 - y + mu2));

  return all_entries;
}


// [[Rcpp::export]]
arma::mat grad_vmcos_all_comp(arma::mat data, arma::mat par_mat,
                              arma::vec pi, arma::mat uni_rand, int ncores = 1)
{
  int n = data.n_rows, K = pi.size(), j;
  arma::vec l_c_vmcos = log_const_vmcos_all(par_mat, uni_rand, ncores);
  arma::vec c_vmcos = arma::exp(l_c_vmcos), log_pi = log(pi);

  stdvec_vec del_const_vmcos_all(K);
  stdvec_vec par_mat_cols(K);

  for(j = 0; j < K; j++) {
    del_const_vmcos_all[j] = d_const_vmcos(par_mat.col(j), uni_rand, ncores);
    par_mat_cols[j] = par_mat.col(j);
  }

  stdvec_mat grad_sum_part(ncores);
  for(int g = 0; g < ncores; g++)
    grad_sum_part[g] = arma::zeros(5, K);

  double denom, temp;

#pragma omp parallel for   private(j, denom, temp) num_threads(ncores)
  for(int i = 0; i < n; i++) {
    arma::mat grad_temp(5, K);
    int g = get_thread_num_final();
    // int g = 0;
    denom = 0;
    for(j = 0; j < K; j++) {
      temp = exp(ldcosnum(data(i, 0), data(i, 1), par_mat_cols[j]) - l_c_vmcos[j] + log_pi[j]);
      denom += temp;
      grad_temp.col(j) = temp*grad_log_vmcos_one_comp_i(data(i, 0), data(i, 1), par_mat_cols[j], c_vmcos[j], del_const_vmcos_all[j]);
    }
    grad_sum_part[g] += grad_temp/denom;
  }

  arma::mat grad_sum = arma::zeros(5, K);
  for(int g = 0; g < ncores; g++)
    grad_sum += grad_sum_part[g];

  return (grad_sum);
}



// [[Rcpp::export]]
double vmcosmix(double x, double y, arma::mat par, arma::vec pi, arma::vec log_c_von)
{
  double res = 0;
  int K = par.n_cols;

  for(int j = 0; j < K; j++)
    res += exp(ldcosnum(x, y, par.col(j)) - log_c_von[j]) * pi[j];
  return(res);
} //likelihood contribution (density) of a single point in a mixture model

// [[Rcpp::export]]
arma::vec vmcosmix_manyx(arma::mat x, arma::mat par, arma::vec pi, arma::vec log_c_von)
{
  int n = x.n_rows;
  arma::vec result(n);
  for(int i = 0; i < n; i++)
    result[i] = vmcosmix(x(i,0), x(i,1), par, pi, log_c_von);
  return(result);
}
